﻿Imports System.Collections.ObjectModel
Imports System.Windows.Forms
Imports System.ComponentModel
Imports C1.WPF.C1Chart

Class MainWindow

    Public Property DataItems() As ObservableCollection(Of DataItem)
        Get
            Return m_DataItems
        End Get
        Private Set(ByVal value As ObservableCollection(Of DataItem))
            m_DataItems = value
        End Set
    End Property
    Private m_DataItems As ObservableCollection(Of DataItem)
    Public ItemsDictionary As New Dictionary(Of Integer, DataItem)()

    Private Shared ReadOnly RandomGenerator As New Random()

    Private Const NumPoints As Integer = 3000

    Private ReadOnly _timer As System.Threading.Timer

    Public Sub New()
        InitializeComponent()

        DataContext = Me

        DataItems = New ObservableCollection(Of DataItem)()

        InitializeData()

        AddTooltipLayer(_scatterPlot)

        _timer = New System.Threading.Timer(AddressOf OnTimerUpdate, Nothing, 10000, 10000)
    End Sub

    Private Sub AddTooltipLayer(ByVal chart As C1Chart)
        Dim pnl = New ChartPanel()

        Dim obj = New ChartPanelObject()
        obj.HorizontalAlignment = Windows.HorizontalAlignment.Right
        obj.VerticalAlignment = Windows.VerticalAlignment.Bottom

        Dim bdr = New Border()
        bdr.Background = New SolidColorBrush(Colors.Blue)
        bdr.Opacity = 0.8
        bdr.BorderBrush = New SolidColorBrush(Colors.Blue)
        bdr.BorderThickness = New Thickness(1, 1, 3, 3)
        bdr.CornerRadius = New CornerRadius(6, 6, 0, 6)
        bdr.Padding = New Thickness(3)

        Dim sp = New StackPanel()

        Dim tb1 = New TextBlock()
        Dim bind1 As System.Windows.Data.Binding = New System.Windows.Data.Binding()
        bind1.Source = obj
        bind1.StringFormat = "x={0:#.##}"
        bind1.Path = New PropertyPath("DataPoint.X")
        tb1.SetBinding(TextBlock.TextProperty, bind1)

        Dim tb2 = New TextBlock()
        Dim bind2 As System.Windows.Data.Binding = New System.Windows.Data.Binding()
        bind2.Source = obj
        bind2.StringFormat = "y={0:#.##}"
        bind2.Path = New PropertyPath("DataPoint.Y")
        tb2.SetBinding(TextBlock.TextProperty, bind2)

        sp.Children.Add(tb1)
        sp.Children.Add(tb2)

        bdr.Child = sp

        obj.Content = bdr
        obj.DataPoint = New Point()
        obj.Action = ChartPanelAction.MouseMove
        obj.Attach = ChartPanelAttach.DataXY

        pnl.Children.Add(obj)

        chart.View.Layers.Add(pnl)
    End Sub

    Private Sub OnTimerUpdate(ByVal state As Object)
        For i As Integer = 0 To NumPoints - 1
            Dim item As New DataItem
            If ItemsDictionary.TryGetValue(i, item) Then
                item("XValue") = RandomGenerator.NextDouble()
                item("YValue") = RandomGenerator.NextDouble()
            End If
        Next

        ToolBox.DispatchToUI(Sub()
                                 DataItems = New ObservableCollection(Of DataItem)(ItemsDictionary.Values)
                                 OnPropertyChanged("DataItems")

                             End Sub)
    End Sub

    Public Event PropertyChanged As PropertyChangedEventHandler

    Protected Overloads Sub OnPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub InitializeData()
        For i As Integer = 0 To NumPoints - 1
            Dim item = New DataItem()
            item("XValue") = RandomGenerator.NextDouble()

            item("YValue") = RandomGenerator.NextDouble()

            ItemsDictionary(i) = item
            DataItems.Add(item)
        Next
    End Sub

End Class
